from dataclasses import dataclass
from typing import Optional

from delphinium.entities.base import HeliotropeEntity


@dataclass
class Language(HeliotropeEntity):
    galleryid: Optional[int]
    language_localname: str
    name: str
    url: str



from dataclasses import dataclass

from delphinium.entities.base import HeliotropeEntity
from delphinium.entities.file import File


@dataclass
class ResolvedImage(HeliotropeEntity):
    url: str
    file: File



from dataclasses import dataclass

from delphinium.entities.base import HeliotropeEntity
from delphinium.entities.info import Info


@dataclass
class SearchResultDTO(HeliotropeEntity):
    results: list[Info]
    count: int



from delphinium.entities.artist import Artist
from delphinium.entities.character import Character
from delphinium.entities.file import File
from delphinium.entities.galleryinfo import Galleryinfo
from delphinium.entities.group import Group
from delphinium.entities.info import Info
from delphinium.entities.language import Language
from delphinium.entities.parody import Parody
from delphinium.entities.resolved_image import ResolvedImage
from delphinium.entities.tag import Tag

__all__ = [
    "Artist",
    "Character",
    "File",
    "ResolvedImage",
    "Galleryinfo",
    "Group",
    "Info",
    "Language",
    "Parody",
    "Tag",
]



from typing import Any, Optional

from aiohttp import ClientSession

from delphinium.dtos.list import ListResultDTO
from delphinium.dtos.search import SearchResultDTO
from delphinium.dtos.thumbnail import Size
from delphinium.entities import *
from delphinium.entities.resolved_image import ResolvedImage
from delphinium.entities.tags import Tags
from delphinium.http import DelphiniumHTTP


class Delphinium:
    def __init__(
        self, base_url: str, client_session: Optional[ClientSession] = None
    ) -> None:
        self.http = DelphiniumHTTP(base_url, client_session)

    async def get_galleryinfo(self, id: int) -> Galleryinfo:
        data = await self.http.get_galleryinfo(id)
        return Galleryinfo.from_dict(data)

    async def get_image(self, id: int) -> list[ResolvedImage]:
        data = await self.http.get_image(id)
        return [ResolvedImage.from_dict(item) for item in data]

    async def get_info(self, id: int) -> Info:
        data = await self.http.get_info(id)
        return Info.from_dict(data)

    async def get_list(self, index: int) -> ListResultDTO:
        data = await self.http.get_list(index)
        return ListResultDTO.from_dict(data)

    async def get_tags(self) -> Tags:
        data = await self.http.get_tags()
        return Tags.from_dict(data)

    async def get_thumbnail(
        self, id: int, size: Size = Size.SMALL, single: bool = True
    ) -> list[ResolvedImage]:
        data = await self.http.get_thumbnail(id, size, single)
        return [ResolvedImage.from_dict(item) for item in data]

    async def post_random(self, query: list[str]) -> Info:
        data = await self.http.post_random(query)
        return Info.from_dict(data)

    async def post_search(self, query: list[str], offset: int = 0) -> SearchResultDTO:
        data = await self.http.post_search(query, offset)
        return SearchResultDTO.from_dict(data)

    async def close(self) -> None:
        if self.http.client_session:
            await self.http.client_session.close()

    async def __aenter__(self) -> "Delphinium":
        await self.http.__aenter__()
        return self

    async def __aexit__(self, *args: Any) -> None:
        await self.http.__aexit__(*args)



from __future__ import annotations

from dataclasses import dataclass
from datetime import date, datetime
from typing import Optional

from delphinium.entities.artist import Artist
from delphinium.entities.base import HeliotropeEntity
from delphinium.entities.character import Character
from delphinium.entities.file import File
from delphinium.entities.group import Group
from delphinium.entities.language import Language
from delphinium.entities.parody import Parody
from delphinium.entities.tag import Tag


@dataclass
class Galleryinfo(HeliotropeEntity):
    date: datetime
    galleryurl: str
    id: int
    japanese_title: Optional[str]
    language_localname: str
    language_url: str
    language: str
    title: str
    type: str
    video: Optional[str]
    videofilename: Optional[str]
    blocked: bool
    datepublished: Optional[date]
    artists: list[Artist]
    characters: list[Character]
    files: list[File]
    groups: list[Group]
    languages: list[Language]
    parodys: list[Parody]
    related: list[int]
    scene_indexes: list[int]
    tags: list[Tag]



from dataclasses import dataclass, field

from delphinium.entities.base import HeliotropeEntity


@dataclass
class Tag(HeliotropeEntity):
    tag: str
    url: str
    female: bool = field(default=False)
    male: bool = field(default=False)



from typing import Literal, NamedTuple


class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: Literal["alpha", "beta", "candidate", "final"]
    serial: int


version_info = VersionInfo(major=1, minor=6, micro=0, releaselevel="final", serial=0)

__version__ = f"{version_info.major}.{version_info.minor}.{version_info.micro}"

if version_info.releaselevel != "final":
    __version__ = f"{__version__}-{version_info.releaselevel}.{version_info.serial}"



from dataclasses import dataclass

from delphinium.entities.base import HeliotropeEntity


@dataclass
class Tags(HeliotropeEntity):
    artists: list[str]
    groups: list[str]
    series: list[str]
    characters: list[str]
    tag: list[str]
    male: list[str]
    female: list[str]
    type: list[str]
    language: list[str]



from dataclasses import dataclass
from datetime import date, datetime
from typing import Any, Mapping, Self, Union, get_args, get_origin, get_type_hints


@dataclass
class HeliotropeEntity:
    @classmethod
    def from_dict(cls, data: Mapping[str, Any]) -> Self:
        converted_data: dict[str, Any] = {}
        type_hints = get_type_hints(cls)

        for key, value in data.items():
            type_ = type_hints.get(key)

            if type_ is None:
                raise ValueError(f"Key {key} not found in {cls.__name__}")

            if type_ is datetime:
                value = datetime.fromisoformat(value).replace(tzinfo=None)
            # Union type
            elif origin_type := get_origin(type_):
                # Optional type
                if origin_type is Union and type(None) in get_args(type_):
                    if value is not None:
                        arg_type = get_args(type_)[0]
                        if arg_type is date:
                            value = date.fromisoformat(value)
                        else:
                            value = arg_type(value)
                elif origin_type is list:
                    arg_type = get_args(type_)[0]
                    if issubclass(arg_type, HeliotropeEntity):
                        if value is None:
                            value = []
                        else:
                            value = [arg_type.from_dict(v) for v in value]
                    elif arg_type is int:
                        value = [int(v) for v in value]
            elif issubclass(type_, HeliotropeEntity):
                value = type_.from_dict(value)
            elif type_ is bool:
                if value in ["0", 0, None, ""]:
                    value = False
                if value in ["1", 1]:
                    value = True
            else:
                value = type_(value)
            converted_data[key] = value
        return cls(**converted_data)



from platform import python_version
from types import TracebackType
from typing import Any, Literal, Optional

from aiohttp import ClientSession
from aiohttp import __version__ as aiohttp_version

from delphinium import __version__ as delphinium_version
from delphinium.dtos.thumbnail import Size
from delphinium.error import DelphiniumHTTPError
from delphinium.types import (
    HeliotropeGalleryinfoJSON,
    HeliotropeInfoJSON,
    HeliotropeListResultDTOJSON,
    HeliotropeResolvedImageJSON,
    HeliotropeSearchResultDTOJSON,
    HeliotropeTagsJSON,
)


class DelphiniumHTTP:
    UA = f"Delphinium (https://github.com/Saebasol/Delphinium {delphinium_version}) Python/{python_version()} aiohttp/{aiohttp_version}"

    def __init__(
        self, base_url: str, client_session: Optional[ClientSession] = None
    ) -> None:
        self.base_url = base_url.rstrip("/")
        self.client_session = client_session

    async def request(
        self,
        method: Literal["GET", "POST"],
        path: str,
        json: Optional[dict[str, Any]] = None,
    ) -> Any:
        url = self.base_url + "/api/hitomi" + path

        if not self.client_session:
            self.client_session = ClientSession()

        async with self.client_session.request(
            method, url, json=json, headers={"user-agent": self.UA}
        ) as resp:
            body = await resp.json()

            if resp.status != 200:
                raise DelphiniumHTTPError(body["message"])

            return body

    async def get_galleryinfo(self, id: int) -> HeliotropeGalleryinfoJSON:
        return await self.request("GET", f"/galleryinfo/{id}")

    async def get_image(self, id: int) -> list[HeliotropeResolvedImageJSON]:
        return await self.request("GET", f"/image/{id}")

    async def get_info(self, id: int) -> HeliotropeInfoJSON:
        return await self.request("GET", f"/info/{id}")

    async def get_list(self, index: int) -> HeliotropeListResultDTOJSON:
        return await self.request("GET", f"/list/{index}")

    async def get_tags(self) -> HeliotropeTagsJSON:
        return await self.request("GET", "/tags")

    async def get_thumbnail(
        self,
        id: int,
        size: Size,
        single: bool,
    ) -> list[HeliotropeResolvedImageJSON]:
        single_str = "true" if single else "false"
        return await self.request(
            "GET", f"/thumbnail/{id}?size={size.value}&single={single_str}"
        )

    async def post_random(self, query: list[str]) -> HeliotropeInfoJSON:
        return await self.request("POST", "/random", {"query": query})

    async def post_search(
        self, query: list[str], offset: int
    ) -> HeliotropeSearchResultDTOJSON:
        return await self.request(
            "POST",
            f"/search?offset={offset}",
            {
                "query": query,
            },
        )

    async def __aenter__(self) -> "DelphiniumHTTP":
        return self

    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        if self.client_session:
            await self.client_session.close()
